;
(function (jQuery) {
    jQuery.fn.dpTileSlider = function (options) {
        var jQueryslider = this;

        // Define the slider options
        var jQueryoptions = options || {};
        //TODO: public method
        //TODO: hide slider before fully load

        jQueryoptions = {
            /**
             * Time delay before start animation on each slide.
             * This support you to setup your additional content before animation started
             */
            timeout: ((options && options.timeout) ? options.timeout : 50),
            // If set to true the slider will auto play
            auto: ((options && options.auto) ? options.auto : 'false'),
            // Toggle on/off the slider navigation and/or pager
            nav: ((options && options.nav) ? options.nav : 'true'),
            pager: ((options && options.pager) ? options.pager : 'true'),
            // Callback function fire when slider first start
            start: ((options && options.start) ? options.start : function () {}),
            // Callback function fire before/after each slide start to animate
            animBefore: ((options && options.animBefore) ? options.animBefore : function () {}),
            animAfter: ((options && options.animAfter) ? options.animAfter : function () {})
        };

        // Get & set slide selectors
        var dpSlides = jQueryslider.children(),
                dpNext = jQuery('<a class="next">Next<span></span></a>'),
                dpPrev = jQuery('<a class="prev">Previous<span></span></a>'),
                pagers = makePager();

        // Array of animations type
        var anims = {
            scale: 'scale-anim',
            translate: 'translate-clockwise-anim',
            split: 'split-anim',
            split2: 'split-anim-sync',
            fall: 'fall-anim',
            rotate3D: 'rotate3D-anim',
            rotate3D2: 'rotate3D-anim-sync',
            rotate: 'rotate-anim'
        };

        // This indicate if the slider is animated
        var isAnim = false;

        // Initialize the slider
        function initSlider() {
            // Setup doms for slider
              
      
            initDoms(dpSlides);
    
            // Resize images when windows resize
            jQuery(window).on('resize', function () {
                var imgs = dpSlides.find('img[role=banner]');
                imgs.each(function () {
                    sizingImgs(jQuery(this));
                });
            });

            /**
             * Callback function fire when first starting slider,
             * Associate with active slide
             */
            jQueryoptions.start.call(dpSlides.filter('.active'));

            // When next button is clicked
            dpNext.on('click', function (e) {
                e.preventDefault();
                playNext('next');
            });

            // When previous button is clicked
            dpPrev.on('click', function (e) {
                e.preventDefault();
                playNext('prev');
            });

            //FUTURE: add swipe guestures

            // When every pager button is clicked
            pagers.each(function () {
                jQuery(this).find('a').each(function () {
                    jQuery(this).on('click', function (e) {
                        e.preventDefault();
                        var idx = jQuery(this).data('slide');
                        playNext('shuff', dpSlides.filter('[data-slide=' + idx + ']'));
                    });
                });
            });

            /**
             * Set auto slide on/off,
             * when no navigation & pager are visible the auto slide is on
             */
            if (jQueryoptions.auto === 'true' ||
                    (jQueryoptions.nav === 'false' && jQueryoptions.pager === 'false')) {
                autoSlide();
            }
        }

        /**
         * Setup doms elements
         * @param {Array} slides collection of slides
         */
        function initDoms(slides) {
            if (!jQueryslider.hasClass('dp-tile-slider')) {
                jQueryslider.addClass('dp-tile-slider');
            }

            slides.each(function () {
                if (!jQuery(this).hasClass('dp-slide')) {
                    jQuery(this).addClass('dp-slide');
                }

                var img = jQuery(this).children('a[role=banner]').eq(0);

                var dpTile = jQuery('<div class="dp-tile"></div>'),
                        dpImg = jQuery('<div class="dp-img"></div>');

                dpImg.append(img);
                dpTile.append(dpImg);

                for (var i = 0; i < 4; i++) {
                    var tile = dpTile.clone();
                    tile.prependTo(jQuery(this));
                }
            });

            if (jQueryoptions.nav === 'true') {
                jQueryslider.append(dpNext, dpPrev);
            }

            if (jQueryoptions.pager === 'true') {
                jQueryslider.append(pagers);
            }
        }

        /**
         * Risize the image according to the size of slider container
         * @param {Object} img image to be resized
         */
        function sizingImgs(imgs) {
            imgs.each(function () {
                var winWidth = jQueryslider.width(),
                        winHeight = jQueryslider.height(),
                        imgWidth = this.width,
                        imgHeight = this.height,
                        winPro = winWidth / winHeight,
                        imgPro = imgWidth / imgHeight;

                if (imgPro < winPro) {
                    jQuery(this).addClass('is-landscape');
                } else {
                    jQuery(this).removeClass('is-landscape');
                }
            });
        }

        /**
         * Play the animation && show the next slide
         * @param {String} dir   the direction left/right/shuff
         * @param {Object} slide slide to show next
         */
        function playNext(dir, slide) {
            if (isAnim) {
                return;
            }
            isAnim = true;

            var activeSlide = jQueryslider.children('.dp-slide.active'),
                    nextSlide,
                    dataAnim = activeSlide.data('anim'),
                    random = Math.round(Math.random() * (getObjSize(anims) - 1)),
                    anim = anims.hasOwnProperty(dataAnim) ?
                    anims[ dataAnim ] : getObjValueAt(anims, random);

            // Callback function fire at the begining of each slide
            jQueryoptions.animBefore.call(activeSlide);

            setTimeout(function () {
                if (dir === 'next') {
                    nextSlide = getNextSlide(activeSlide);
                } else if (dir === 'prev') {
                    nextSlide = getPreviousSlide(activeSlide);
                } else if (dir === 'shuff' && slide) {
                    nextSlide = slide;
                }

                if (activeSlide.is(nextSlide)) {
                    isAnim = false;
                    return;
                }

                activeSlide.removeClass('active')
                        .addClass('animated ' + anim)
                        .find('.dp-tile .dp-img')
                        .last()
                        .one('webkitAnimationEnd oanimationend oAnimationEnd msAnimationEnd animationend',
                                function () {
                                    isAnim = false;
                                    activeSlide.removeClass('animated ' + anim);

                                    /**
                                     * Callback fire after each slide animated
                                     */
                                    jQueryoptions.animAfter.call(nextSlide);
                                });

                nextSlide.addClass('active')
                        .one('webkitTransitionEnd otransitionend oTransitionEnd msTransitionEnd transitionend',
                                function () {
                                    var pagerBtns = pagers.find('a');
                                    pagerBtns.filter('[data-slide=' + activeSlide.data('slide') + ']')
                                            .removeClass('active');
                                    pagerBtns.filter('[data-slide=' + nextSlide.data('slide') + ']')
                                            .addClass('active');
                                });
            }, jQueryoptions.timeout);
        }

        /**
         * Get next slide
         * @param   {Object} activeSlide current slide
         * @returns {Object} next slide
         */
        function getNextSlide(activeSlide) {
            var nextSlide = activeSlide.next('.dp-slide');
            if (nextSlide.length <= 0) {
                nextSlide = activeSlide.siblings('.dp-slide').first();
            }

            return nextSlide;
        }

        /**
         * Get previous slide
         * @param   {Object} activeSlide current slide
         * @returns {Object} previous slide
         */
        function getPreviousSlide(activeSlide) {
            var prevSlide = activeSlide.prev('.dp-slide');
            if (prevSlide.length <= 0) {
                prevSlide = activeSlide.siblings('.dp-slide').last();
            }

            return prevSlide;
        }

        /**
         * Get number of properties of the object
         * @param   {Object}   obj Object to be got size
         * @returns {[[Type]]} [[Description]]
         */
        function getObjSize(obj) {
            var size = 0,
                    key;
            for (key in obj) {
                if (obj.hasOwnProperty(key)) {
                    size++;
                }
            }
            return size;
        }

        /**
         * Get property value of the object at the index
         * @param   {Object} obj Object
         * @param   {Number} idx Index of the property
         * @returns {String} Value of the property
         */
        function getObjValueAt(obj, idx) {
            var i = 0,
                    key,
                    val = false;
            for (key in obj) {
                if (obj.hasOwnProperty(key)) {
                    if (i === idx) {
                        val = obj[ key ];
                        break;
                    } else {
                        i++;
                    }
                }
            }

            return val;
        }

        /**
         * Create pager
         * @returns {Object} Pager
         */
        function makePager() {
            var ul = jQuery('<ul/>').addClass('dp-pager'),
                    i = 0;

            for (i = 0; i < dpSlides.length; i++) {
                var li = jQuery('<li></li>'),
                        a = jQuery('<a href="#"></a>');

                a.attr('data-slide', '' + i);
                if (i === 0) {
                    a.addClass('active');
                    dpSlides.eq(i).addClass('active');
                }
                li.append(a);
                ul.append(li);
                dpSlides.eq(i).attr('data-slide', '' + i);
            }

            return ul;
        }

//        var img = jQuery(".screeny"); 
//    img.load(function() {
//    var height = img.height();
//    jQuery("#dp-slider").css("height", height);
//});

  
        /**
         * Auto slide
         */
        function autoSlide() {
            var timeDelay = 2000;
            setTimeout(function () {
                playNext('next');
                setTimeout(function () {
                    autoSlide();
                }, timeDelay);
            }, timeDelay);
        }

        jQuery(window).load(function () {
            var imgs = dpSlides.find('img[role=banner]');
            sizingImgs(imgs);
            imgs.each(function () {
                jQuery(this).show();
            });
        });

        initSlider();
        return jQueryslider;
    };
})(jQuery);

